/**
 * XMLToZip.java
 *
 * It is provided for illustration on as "as is" basis.
 * It should not be considered production ready.
 */

import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.DocumentBuilder;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.NamedNodeMap;
import java.util.logging.Logger;
import java.util.logging.Level;
import java.io.FileOutputStream;
import java.util.zip.ZipOutputStream;
import java.util.zip.ZipEntry;

public class XMLToZip {

    static final String errorMessagePrefix = "XML to Zip error: ";
    static final int bufferSize = 2048;
    
    /**
     * Convert the XML file in the first argument to a ZIP file named in the second.
     * Each "file" in the input must be a single XML document
     * The XML file has the format:
     * <root>
     *   <directory name="">
     *     other directory and file elements
     *   </directory>
     *   <file name="">
     *     XML element to be converted to "file" in compressed output
     *   </file>
     * </root>
     */

    public static void main (String [] args) throws Exception {
        XMLToZip xmlToZip = new XMLToZip();
        xmlToZip.doZip(args[0],args[1]);
    }

    /**
     * Do the ZIP, with two file names
     */
    void doZip(String inFileName,String outFileName) throws Exception {

        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setNamespaceAware(true);
        DocumentBuilder builder = factory.newDocumentBuilder();
        Document inDocument = builder.parse(inFileName);

        ZipOutputStream zos = new ZipOutputStream(new FileOutputStream(outFileName));
        doZip(inDocument.getDocumentElement(),"",zos);
        zos.close();
    }

    /**
     * Do the zip, from an directory element and an output file name
     * Directory prefix is added to each file and directory name
     * At root this is ""
     * At directory XXX, this is "XXX/", i.e. it ends with a slash
     */
    void doZip(Element directoryElement,String directoryPrefix,ZipOutputStream zos) throws Exception {
        NodeList nl = directoryElement.getChildNodes();
        for ( int i = 0 ; i < nl.getLength(); i++ ) {
            if ( nl.item(i).getNodeType() == Node.ELEMENT_NODE ) {
                Element childElement = (Element) nl.item(i);
                if ( childElement.getTagName().equals("file") ) {
                    String name = childElement.getAttribute("name");
                    if ( name == null || name.equals("") ) {
                        String exceptionMessage = errorMessagePrefix + "missing name attribute on file element at level \"" + directoryPrefix + "\".";
                        severe(exceptionMessage);
                        throw new Exception(exceptionMessage);
                    }
                    String fullPath = directoryPrefix + name;
                    Element fileRootElement = getFirstElement(childElement);
                    if ( fileRootElement == null ) {
                        String exceptionMessage = errorMessagePrefix + "no content in file element \"" + fullPath + "\".";
                        severe(exceptionMessage);
                        throw new Exception(exceptionMessage);
                    }
                    zos.putNextEntry(new ZipEntry(fullPath));
                    zos.write(nodeToString(fileRootElement).getBytes());
                } else if ( childElement.getTagName().equals("directory") ) {
                    String name = childElement.getAttribute("name");
                    if ( name == null || name.equals("") ) {
                        String exceptionMessage = errorMessagePrefix + "missing name attribute on directory element at level \"" + directoryPrefix + "\".";
                        severe(exceptionMessage);
                        throw new Exception(exceptionMessage);
                    }
                    doZip(childElement,directoryPrefix + name + "/",zos);
                }
            }
        }
    }

    /**
     * Return the first element within another element.
     */
    Element getFirstElement(Element element) throws Exception {
        NodeList childNodes = element.getChildNodes();
        for ( int i = 0 ; i < childNodes.getLength(); i++ ) {
            if ( childNodes.item(i).getNodeType() == Node.ELEMENT_NODE ) {
                return (Element) childNodes.item(i);
            }
        }
        return null;
    }

    /**
     * Convert a node to an XML string.
     */
    String nodeToString(Node node) {
        StringBuffer sb = new StringBuffer();
        return nodeToString(sb,node).toString();
    }

    /**
     * Append a string representation of a node to a string buffer.
     */
    StringBuffer nodeToString(StringBuffer sb, Node node) {
        switch (node.getNodeType()) {
            case Node.ELEMENT_NODE:
                sb.append('<');
                sb.append(node.getNodeName());
                if ( node.hasAttributes() ) {
                    // Process attributes
                    NamedNodeMap nnm = node.getAttributes();
                    for ( int i = 0 ; i < nnm.getLength(); i++ ) {
                        nodeToString(sb,nnm.item(i));
                    }
                }
                if ( node.hasChildNodes() ) {
                    sb.append('>');
                    // Now process all the other children that aren't attributes
                    NodeList nl = node.getChildNodes();
                    for ( int i = 0 ; i < nl.getLength(); i++ ) {
                        // I don't think children are ever attributes
                        if ( nl.item(i).getNodeType() != Node.ATTRIBUTE_NODE ) {
                            nodeToString(sb,nl.item(i));
                        }
                    }
                    sb.append("</");
                    sb.append(node.getNodeName());
                    sb.append('>');
                } else {
                    sb.append("/>");
                }
                break;
            case Node.ATTRIBUTE_NODE:
                sb.append(' ');
                sb.append(node.getNodeName());
                sb.append("=\"");
                escapeXML(sb,node.getNodeValue());
                sb.append('\"');
                break;
            case Node.TEXT_NODE:
            case Node.CDATA_SECTION_NODE:
                escapeXML(sb,node.getNodeValue());
                break;
            default:
                // Do nothing - not interested
                break;
        }
        return sb;
    }

    /**
     * Append an escaped XML string to a StringBuffer.
     */
    public static StringBuffer escapeXML(StringBuffer sb, String s){

        int p;
        char c;
        if ( s == null ) {
            return sb;
        }
        for ( p = 0 ; p < s.length() ; p++ ) {
            c = s.charAt(p);

            if ( c == '<' ) {
                sb.append("&lt;");
            } else if ( c == '>' ) {
                sb.append("&gt;");
            } else if (  c == '&' ) {
                sb.append("&amp;");
            } else if (  c == '\'' ) {
                sb.append("&apos;");
            } else if (  c == '\"' ) {
                sb.append("&quot;");
            } else if ( c > 127 ) {
                sb.append("&#" + (int)c + ";");
            } else {
                sb.append(c);
            }
        }
        return sb;
    }

    /**
     * Log a severe message
     */
    private void severe(String msg) {
        Logger.getLogger("XMLToZip").log(Level.SEVERE,msg);
    }

}

