<!--
  - Create a directory structure for Excel 2007 from a tabular export.
  -->
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">

<xsl:template match="TableExport">
  <root>
    <xsl:call-template name="rels"/>
    <directory name="xl">
      <xsl:call-template name="xl-rels"/>
      <xsl:call-template name="worksheets"/>
      <xsl:call-template name="styles"/>
      <xsl:call-template name="workbook"/>
    </directory>
    <xsl:call-template name="content-types"/>
  </root>
</xsl:template>

<!-- Produce high-level relationship directory and file -->
<xsl:template name="rels">
  <directory name="_rels">
    <file name=".rels">
      <Relationships xmlns="http://schemas.openxmlformats.org/package/2006/relationships">
        <Relationship Id="rId1" Type="http://schemas.openxmlformats.org/officeDocument/2006/relationships/officeDocument" Target="xl/workbook.xml"/>
      </Relationships>
    </file>
  </directory>
</xsl:template>

<!-- Produce relationship directory inside xl directory -->
<xsl:template name="xl-rels">
  <directory name="_rels">
    <file name="workbook.xml.rels">
      <Relationships xmlns="http://schemas.openxmlformats.org/package/2006/relationships">
        <xsl:for-each select="TableList/Table">
          <Relationship Id="rId{position()}" Type="http://schemas.openxmlformats.org/officeDocument/2006/relationships/worksheet"
            Target="worksheets/sheet{position()}.xml"/>
        </xsl:for-each>
        <Relationship Id="rIdStyles" Type="http://schemas.openxmlformats.org/officeDocument/2006/relationships/styles" Target="styles.xml" />
      </Relationships>
    </file>
  </directory>
</xsl:template>

<!-- Create each of the worksheets -->
<xsl:template name="worksheets">
  <directory name="worksheets">
    <xsl:apply-templates select="TableList/Table"/>
  </directory>
</xsl:template>

<!-- Create one worksheet -->
<xsl:template match="Table">
  <file name="sheet{position()}.xml">
    <worksheet xmlns="http://schemas.openxmlformats.org/spreadsheetml/2006/main">
      <cols>
        <xsl:apply-templates select="ColumnList/Column" mode="colWidth"/>
      </cols>
      <sheetData>
        <row r="1">
          <xsl:apply-templates select="ColumnList/Column" mode="heading"/>
        </row>
        <xsl:apply-templates select="RowList/Row" mode="data"/>
      </sheetData>
    </worksheet>
  </file>
</xsl:template>

<!-- Default column width specification -->
<xsl:template match="Column" mode="colWidth">
  <col min="{position()}" max="{position()}" width="20" customWidth="1"  xmlns="http://schemas.openxmlformats.org/spreadsheetml/2006/main"/>
</xsl:template>

<!-- Wider columns for text and markup -->
<xsl:template match="Column[dataType = 'text' or dataType = 'markup']" mode="colWidth">
  <col min="{position()}" max="{position()}" width="40" customWidth="1"  xmlns="http://schemas.openxmlformats.org/spreadsheetml/2006/main"/>
</xsl:template>

<!-- Heading row -->
<xsl:template match="Column" mode="heading">
  <xsl:variable name="columnLetter">
    <xsl:call-template name="calculate-column-letter">
      <xsl:with-param name="columnNumber" select="position()"/>
    </xsl:call-template>
  </xsl:variable>
  <c r="{$columnLetter}1" s="2" t="inlineStr" xmlns="http://schemas.openxmlformats.org/spreadsheetml/2006/main">
    <is>
      <t><xsl:value-of select="columnName"/></t>
    </is>
  </c>
</xsl:template>

<!-- 
  - Data row
  - To cope with mis-ordered fields and null data, we iterate over the columns
  - and output the matching fields.
  -->

<xsl:template match="Row" mode="data">
  <xsl:variable name="rowNumber" select="position() + 1"/>
  <xsl:variable name="row" select="."/>
  <row xmlns="http://schemas.openxmlformats.org/spreadsheetml/2006/main" r="{$rowNumber}">
    <xsl:for-each select="../../ColumnList/Column">
      <xsl:variable name="columnName" select="columnName"/>
      <xsl:variable name="columnNumber" select="position()"/>
      <xsl:variable name="columnLetter">
        <xsl:call-template name="calculate-column-letter">
          <xsl:with-param name="columnNumber" select="$columnNumber"/>
        </xsl:call-template>
      </xsl:variable>
      <xsl:variable name="dataType" select="dataType"/>
      <xsl:variable name="field" select="$row/FieldList/Field[columnName = $columnName]"/>
      <xsl:if test="$field">
        <xsl:choose>
          <!-- Number -->
          <xsl:when test="$dataType = 'number'">
            <c r="{$columnLetter}{$rowNumber}" t="n" s="1">
              <v>
                <xsl:value-of select="$field/value"/>
              </v>
            </c>
          </xsl:when>
          <!-- Timestamp -->
          <xsl:when test="$dataType = 'timestamp'">
            <c r="{$columnLetter}{$rowNumber}" t="n" s="4">
              <f>DATEVALUE("<xsl:value-of select="substring($field/value,1,10)"/>")+TIMEVALUE("<xsl:value-of select="substring($field/value,12)"/>")</f>
            </c>
          </xsl:when>
          <!-- text and markup might well be multi-line, so we wrap the text -->
          <xsl:when test="$dataType = 'markup' or $dataType = 'text'">
            <c r="{$columnLetter}{$rowNumber}" t="inlineStr" s="3">
              <is>
                <t>
                  <xsl:value-of select="$field/value"/>
                </t>
              </is>
            </c>
          </xsl:when>
          <!-- Simple string (default). -->
          <xsl:otherwise>
            <c r="{$columnLetter}{$rowNumber}" t="inlineStr" s="1">
              <is>
                <t>
                  <xsl:value-of select="$field/value"/>
                </t>
              </is>
            </c>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:if>
    </xsl:for-each>
  </row>
</xsl:template>

<!-- Calculate a column letter -->
<xsl:template name="calculate-column-letter">
  <xsl:param name="columnNumber"/>
  <xsl:choose>
    <xsl:when test="$columnNumber &lt;= 26">
      <xsl:value-of select="substring('ABCDEFGHIJKLMNOPQRSTUVWXYZ',$columnNumber,1)"/>
    </xsl:when>
    <xsl:otherwise>
      <xsl:call-template name="calculate-column-letter">
        <xsl:with-param name="columnNumber" select="floor($columnNumber div 26)"/>
      </xsl:call-template>
      <xsl:call-template name="calculate-column-letter">
        <xsl:with-param name="columnNumber" select="$columnNumber mod 26"/>
      </xsl:call-template>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!-- Standard set of styles -->
<xsl:template name="styles">
  <file name="styles.xml">
    <styleSheet xmlns="http://schemas.openxmlformats.org/spreadsheetml/2006/main">
      <fonts count="2">
        <!-- font 0 - normal -->
        <font>
          <sz val="11"/>
          <color theme="1"/>
          <name val="Calibri"/>
          <family val="2"/>
          <scheme val="minor"/>
        </font>
        <!-- font 1 - bold -->
        <font>
          <sz val="11"/>
          <color theme="1"/>
          <name val="Calibri"/>
          <family val="2"/>
          <scheme val="minor"/>
          <b/>
        </font>
      </fonts>
      <fills count="1">
        <fill>
          <patternFill patternType="none"/>
        </fill>
      </fills>
      <borders count="1">
        <border>
          <left/>
          <right/>
          <top/>
          <bottom/>
          <diagonal/>
        </border>
      </borders>
      <cellXfs count="2">
        <!-- default -->
        <xf/>
        <!-- style 1 - vertical align (normal) -->
        <xf applyAlignment="1">
          <alignment vertical="top"/>
        </xf>
        <!-- style 2 - bold -->
        <xf applyFont="1" fontId="1" applyAlignment="1">
          <alignment vertical="top"/>
        </xf>
        <!-- style 3 - multi-line -->
        <xf applyAlignment="1">
          <alignment wrapText="1" vertical="top"/>
        </xf>
        <!-- style 4 - timestamp -->
        <xf numFmtId="22" fontId="0" fillId="0" borderId="0" xfId="0" applyNumberFormat="1" applyAlignment="1">
          <alignment vertical="top"/>
        </xf>
      </cellXfs>
    </styleSheet>

  </file>
</xsl:template>

<!-- Create the workbooks -->
<xsl:template name="workbook">
  <file name="workbook.xml">
    <workbook xmlns="http://schemas.openxmlformats.org/spreadsheetml/2006/main"
      xmlns:r="http://schemas.openxmlformats.org/officeDocument/2006/relationships">
      <sheets>
        <xsl:for-each select="TableList/Table">
          <sheet name="{tableName}" sheetId="{position()}" r:id="rId{position()}"/>
        </xsl:for-each>
      </sheets>
    </workbook>
  </file>
</xsl:template>

<!-- Create the content types -->
<xsl:template name="content-types">
  <file name="[Content_Types].xml">
    <Types xmlns="http://schemas.openxmlformats.org/package/2006/content-types">
      <Default Extension="rels" ContentType="application/vnd.openxmlformats-package.relationships+xml"/>
      <Default Extension="xml" ContentType="application/xml"/>
      <Override PartName="/xl/workbook.xml" ContentType="application/vnd.openxmlformats-officedocument.spreadsheetml.sheet.main+xml"/>
      <xsl:for-each select="TableList/Table">
        <Override PartName="/xl/worksheets/sheet{position()}.xml" ContentType="application/vnd.openxmlformats-officedocument.spreadsheetml.worksheet+xml"/>
      </xsl:for-each>
      <Override PartName="/xl/styles.xml" ContentType="application/vnd.openxmlformats-officedocument.spreadsheetml.styles+xml"/>
    </Types>
  </file>
</xsl:template>

</xsl:stylesheet>
